﻿using System;
using FluentAssertions;
using NUnit.Framework;
using UnityEngine;

namespace Magify.Tests
{
    internal partial class MagifyClientTests
    {
        public class Migration : MagifyClientTestsPart
        {
            [Test]
            public void WhenThereIsMigrationData_ThenMigrationSuccessful()
            {
                // Arrange
                var clientId = Guid.NewGuid().ToString();
                const string firstVersion = "0.0.1";
                using var magifyClient = CreateDefault();
                var generalPrefs = GetGeneralPrefs(magifyClient);

                // Act
                GetPlatformAPI(magifyClient).CustomMigrationData = new MigrationData
                {
                    ClientId = clientId,
                    FirstInstalledVersion = firstVersion,
                };
                magifyClient.InitializeSdk();

                // Assert
                generalPrefs.ClientId.Value.Should()!.Be(clientId);
                generalPrefs.MigratedUser.Should()!.Be(true);
                generalPrefs.FirstInstalledVersion.Should()!.Be(firstVersion);
            }

            [Test]
            public void WhenThereIsNoMigrationData_ThenMigrationSkipped()
            {
                // Arrange
                using var magifyClient = CreateDefault();
                var generalPrefs = GetGeneralPrefs(magifyClient);

                // Act
                magifyClient.InitializeSdk();

                // Assert
                generalPrefs.ClientId.Value.Should()!.NotBeNull();
                generalPrefs.MigratedUser.Should()!.Be(false);
                generalPrefs.FirstInstalledVersion.Should()!.Be(Application.version);
            }
        }
    }
}